/* xl_cheeselooks theme engine
 * Copyright (C) 2005 Richard Stellingwerff
 * Copyright (C) 2007 Benjamin Berg
 * Copyright (C) 2007 Andrea Cimitan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Written by Owen Taylor <otaylor@redhat.com>
 * and by Alexander Larsson <alexl@redhat.com>
 * Modified by Richard Stellingwerff <remenic@gmail.com>
 * Modified by Kulyk Nazar <schamane@myeburg.net>
 */

#include <string.h>
#include <widget-information.h>
#include "xl_cheeselooks_style.h"
#include "xl_cheeselooks_rc_style.h"

#include "animation.h"

static void      xl_cheeselooks_rc_style_init         (xl_cheeselooksRcStyle      *style);
#ifdef HAVE_ANIMATION
static void      xl_cheeselooks_rc_style_finalize     (GObject                *object);
#endif
static void      xl_cheeselooks_rc_style_class_init   (xl_cheeselooksRcStyleClass *klass);
static GtkStyle *xl_cheeselooks_rc_style_create_style (GtkRcStyle             *rc_style);
static guint     xl_cheeselooks_rc_style_parse        (GtkRcStyle             *rc_style,
                                                   GtkSettings            *settings,
                                                   GScanner               *scanner);
static void      xl_cheeselooks_rc_style_merge        (GtkRcStyle             *dest,
                                                   GtkRcStyle             *src);


static GtkRcStyleClass *xl_cheeselooks_parent_rc_class;

GType xl_cheeselooks_type_rc_style = 0;

enum
{
	TOKEN_FOCUSCOLOR = G_TOKEN_LAST + 1,
	TOKEN_SCROLLBARCOLOR,
	TOKEN_COLORIZESCROLLBAR,
	TOKEN_CONTRAST,
	TOKEN_SUNKENMENU,
	TOKEN_PROGRESSBARSTYLE,
	TOKEN_RELIEFSTYLE,
	TOKEN_MENUBARSTYLE,
	TOKEN_TOOLBARSTYLE,
	TOKEN_MENUITEMSTYLE,
	TOKEN_LISTVIEWITEMSTYLE,
	TOKEN_ANIMATION,
	TOKEN_STYLE,
	TOKEN_RADIUS,
	TOKEN_HINT,

	TOKEN_CLASSIC,
	TOKEN_GLOSSY,
	TOKEN_DARK,
	TOKEN_GUMMY,

	TOKEN_TRUE,
	TOKEN_FALSE,

	TOKEN_LAST
};

static gchar* xl_cheeselooks_rc_symbols =
	"focus_color\0"
	"scrollbar_color\0"
	"colorize_scrollbar\0"
	"contrast\0"
	"sunkenmenubar\0"
	"progressbarstyle\0"
	"reliefstyle\0"
	"menubarstyle\0"
	"toolbarstyle\0"
	"menuitemstyle\0"
	"listviewitemstyle\0"
	"animation\0"
	"style\0"
	"radius\0"
	"hint\0"

	"CLASSIC\0"
	"GLOSSY\0"
	"DARK\0"
	"GUMMY\0"

	"TRUE\0"
	"FALSE\0";

void
xl_cheeselooks_rc_style_register_type (GTypeModule *module)
{
	static const GTypeInfo object_info =
	{
		sizeof (xl_cheeselooksRcStyleClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) xl_cheeselooks_rc_style_class_init,
		NULL,           /* class_finalize */
		NULL,           /* class_data */
		sizeof (xl_cheeselooksRcStyle),
		0,              /* n_preallocs */
		(GInstanceInitFunc) xl_cheeselooks_rc_style_init,
		NULL
	};

	xl_cheeselooks_type_rc_style = g_type_module_register_type (module,
	                                                        GTK_TYPE_RC_STYLE,
	                                                        "xl_cheeselooksRcStyle",
	                                                        &object_info, 0);
}

static void
xl_cheeselooks_rc_style_init (xl_cheeselooksRcStyle *xl_cheeselooks_rc)
{
	xl_cheeselooks_rc->style = CL_STYLE_CLASSIC;

	xl_cheeselooks_rc->flags = 0;

	xl_cheeselooks_rc->contrast = 1.0;
	xl_cheeselooks_rc->reliefstyle = 0;
	xl_cheeselooks_rc->menubarstyle = 0;
	xl_cheeselooks_rc->toolbarstyle = 0;
	xl_cheeselooks_rc->animation = FALSE;
	xl_cheeselooks_rc->colorize_scrollbar = FALSE;
	xl_cheeselooks_rc->radius = 3.0;
	xl_cheeselooks_rc->hint = 0;
}

#ifdef HAVE_ANIMATION
static void
xl_cheeselooks_rc_style_finalize (GObject *object)
{
	/* cleanup all the animation stuff */
	xl_cheeselooks_animation_cleanup ();

	if (G_OBJECT_CLASS (xl_cheeselooks_parent_rc_class)->finalize != NULL)
		G_OBJECT_CLASS (xl_cheeselooks_parent_rc_class)->finalize(object);
}
#endif


static void
xl_cheeselooks_rc_style_class_init (xl_cheeselooksRcStyleClass *klass)
{
	GtkRcStyleClass *rc_style_class = GTK_RC_STYLE_CLASS (klass);
#ifdef HAVE_ANIMATION
	GObjectClass    *g_object_class = G_OBJECT_CLASS (klass);
#endif

	xl_cheeselooks_parent_rc_class = g_type_class_peek_parent (klass);

	rc_style_class->parse = xl_cheeselooks_rc_style_parse;
	rc_style_class->create_style = xl_cheeselooks_rc_style_create_style;
	rc_style_class->merge = xl_cheeselooks_rc_style_merge;

#ifdef HAVE_ANIMATION
	g_object_class->finalize = xl_cheeselooks_rc_style_finalize;
#endif
}

static guint
xl_cheeselooks_gtk2_rc_parse_boolean (GtkSettings *settings,
                                  GScanner     *scanner,
                                  gboolean *retval)
{
	guint token;
	token = g_scanner_get_next_token(scanner);

	token = g_scanner_get_next_token(scanner);
	if (token != G_TOKEN_EQUAL_SIGN)
	   return G_TOKEN_EQUAL_SIGN;

	token = g_scanner_get_next_token(scanner);
	if (token == TOKEN_TRUE)
	   *retval = TRUE;
	else if (token == TOKEN_FALSE)
	   *retval = FALSE;
	else
	   return TOKEN_TRUE;

	return G_TOKEN_NONE;
}

static guint
xl_cheeselooks_gtk2_rc_parse_color(GtkSettings  *settings,
                               GScanner     *scanner,
                               GtkRcStyle   *style,
                               GdkColor     *color)
{
	guint token;

	/* Skip 'blah_color' */
	token = g_scanner_get_next_token(scanner);

	token = g_scanner_get_next_token(scanner);
	if (token != G_TOKEN_EQUAL_SIGN)
	   return G_TOKEN_EQUAL_SIGN;

	return gtk_rc_parse_color_full (scanner, style, color);
}

static guint
xl_cheeselooks_gtk2_rc_parse_double (GtkSettings  *settings,
                                 GScanner     *scanner,
                                 gdouble      *val)
{
	guint token;

	/* Skip 'blah' */
	token = g_scanner_get_next_token(scanner);

	token = g_scanner_get_next_token(scanner);
	if (token != G_TOKEN_EQUAL_SIGN)
	   return G_TOKEN_EQUAL_SIGN;

	token = g_scanner_get_next_token(scanner);
	if (token != G_TOKEN_FLOAT)
	   return G_TOKEN_FLOAT;

	*val = scanner->value.v_float;

	return G_TOKEN_NONE;
}

static guint
xl_cheeselooks_gtk2_rc_parse_int (GtkSettings  *settings,
                              GScanner     *scanner,
                              guint8       *progressbarstyle)
{
	guint token;

	/* Skip option name */
	token = g_scanner_get_next_token(scanner);

	token = g_scanner_get_next_token(scanner);
	if (token != G_TOKEN_EQUAL_SIGN)
	   return G_TOKEN_EQUAL_SIGN;

	token = g_scanner_get_next_token(scanner);
	if (token != G_TOKEN_INT)
	   return G_TOKEN_INT;

	*progressbarstyle = scanner->value.v_int;

	return G_TOKEN_NONE;
}

static guint
xl_cheeselooks_gtk2_rc_parse_style (GtkSettings      *settings,
                                GScanner         *scanner,
                                xl_cheeselooksStyles *style)
{
	guint token;

	g_assert (CL_NUM_STYLES == CL_STYLE_GUMMY + 1); /* so that people don't forget ;-) */

	/* Skip 'style' */
	token = g_scanner_get_next_token (scanner);

	token = g_scanner_get_next_token (scanner);
	if (token != G_TOKEN_EQUAL_SIGN)
	   return G_TOKEN_EQUAL_SIGN;

	token = g_scanner_get_next_token (scanner);

	switch (token)
	{
		case TOKEN_CLASSIC:
		   *style = CL_STYLE_CLASSIC;
		   break;
		case TOKEN_GLOSSY:
		   *style = CL_STYLE_GLOSSY;
		   break;
		case TOKEN_DARK:
		   *style = CL_STYLE_DARK;
		   break;
		case TOKEN_GUMMY:
		   *style = CL_STYLE_GUMMY;
		   break;
		default:
		   return TOKEN_CLASSIC;
	}

	return G_TOKEN_NONE;
}

static guint
xl_cheeselooks_gtk2_rc_parse_dummy (GtkSettings      *settings,
                                GScanner         *scanner,
                                gchar            *name)
{
	guint token;

	/* Skip option */
	token = g_scanner_get_next_token (scanner);

	/* print a warning. Isn't there a way to get the string from the scanner? */
	g_scanner_warn (scanner, "xl_cheeselooks configuration option \"%s\" is not supported and will be ignored.", name);

	/* equal sign */
	token = g_scanner_get_next_token (scanner);
	if (token != G_TOKEN_EQUAL_SIGN)
	   return G_TOKEN_EQUAL_SIGN;

	/* eat whatever comes next */
	token = g_scanner_get_next_token (scanner);

	return G_TOKEN_NONE;
}

static guint
xl_cheeselooks_rc_style_parse (GtkRcStyle *rc_style,
                           GtkSettings  *settings,
                           GScanner   *scanner)
{
	static GQuark scope_id = 0;
	xl_cheeselooksRcStyle *xl_cheeselooks_style = xl_cheeselooks_RC_STYLE (rc_style);

	guint old_scope;
	guint token;

	/* Set up a new scope in this scanner. */

	if (!scope_id)
	   scope_id = g_quark_from_string("xl_cheeselooks_theme_engine");

	/* If we bail out due to errors, we *don't* reset the scope, so the
	* error messaging code can make sense of our tokens.
	*/
	old_scope = g_scanner_set_scope(scanner, scope_id);

	/* Now check if we already added our symbols to this scope
	* (in some previous call to xl_cheeselooks_rc_style_parse for the
	* same scanner.
	*/
	if (!g_scanner_lookup_symbol(scanner, xl_cheeselooks_rc_symbols)) {
		gchar *current_symbol = xl_cheeselooks_rc_symbols;
		gint i = G_TOKEN_LAST + 1;

		/* Add our symbols */
		while ((current_symbol[0] != '\0') && (i < TOKEN_LAST)) {
			g_scanner_scope_add_symbol(scanner, scope_id, current_symbol, GINT_TO_POINTER (i));

			current_symbol += strlen(current_symbol) + 1;
			i++;
		}
		g_assert (i == TOKEN_LAST && current_symbol[0] == '\0');
	}

	/* We're ready to go, now parse the top level */

	token = g_scanner_peek_next_token(scanner);
	while (token != G_TOKEN_RIGHT_CURLY)
	{
		switch (token)
		{
			case TOKEN_FOCUSCOLOR:
				token = xl_cheeselooks_gtk2_rc_parse_color (settings, scanner, rc_style, &xl_cheeselooks_style->focus_color);
				xl_cheeselooks_style->flags |= CL_FLAG_FOCUS_COLOR;
				break;
			case TOKEN_SCROLLBARCOLOR:
				token = xl_cheeselooks_gtk2_rc_parse_color (settings, scanner, rc_style, &xl_cheeselooks_style->scrollbar_color);
				xl_cheeselooks_style->flags |= CL_FLAG_SCROLLBAR_COLOR;
				break;
			case TOKEN_COLORIZESCROLLBAR:
				token = xl_cheeselooks_gtk2_rc_parse_boolean (settings, scanner, &xl_cheeselooks_style->colorize_scrollbar);
				xl_cheeselooks_style->flags |= CL_FLAG_COLORIZE_SCROLLBAR;
				break;
			case TOKEN_CONTRAST:
				token = xl_cheeselooks_gtk2_rc_parse_double (settings, scanner, &xl_cheeselooks_style->contrast);
				xl_cheeselooks_style->flags |= CL_FLAG_CONTRAST;
				break;
			case TOKEN_RELIEFSTYLE:
				token = xl_cheeselooks_gtk2_rc_parse_int (settings, scanner, &xl_cheeselooks_style->reliefstyle);
				xl_cheeselooks_style->flags |= CL_FLAG_RELIEFSTYLE;
				break;
			case TOKEN_MENUBARSTYLE:
				token = xl_cheeselooks_gtk2_rc_parse_int (settings, scanner, &xl_cheeselooks_style->menubarstyle);
				xl_cheeselooks_style->flags |= CL_FLAG_MENUBARSTYLE;
				break;
			case TOKEN_TOOLBARSTYLE:
				token = xl_cheeselooks_gtk2_rc_parse_int (settings, scanner, &xl_cheeselooks_style->toolbarstyle);
				xl_cheeselooks_style->flags |= CL_FLAG_TOOLBARSTYLE;
				break;
			case TOKEN_ANIMATION:
				token = xl_cheeselooks_gtk2_rc_parse_boolean (settings, scanner, &xl_cheeselooks_style->animation);
				xl_cheeselooks_style->flags |= CL_FLAG_ANIMATION;
				break;
			case TOKEN_STYLE:
				token = xl_cheeselooks_gtk2_rc_parse_style (settings, scanner, &xl_cheeselooks_style->style);
				xl_cheeselooks_style->flags |= CL_FLAG_STYLE;
				break;
			case TOKEN_RADIUS:
				token = xl_cheeselooks_gtk2_rc_parse_double (settings, scanner, &xl_cheeselooks_style->radius);
				xl_cheeselooks_style->flags |= CL_FLAG_RADIUS;
				break;
			case TOKEN_HINT:
				token = ge_rc_parse_hint (scanner, &xl_cheeselooks_style->hint);
				xl_cheeselooks_style->flags |= CL_FLAG_HINT;
				break;

			/* stuff to ignore */
			case TOKEN_SUNKENMENU:
				token = xl_cheeselooks_gtk2_rc_parse_dummy (settings, scanner, "sunkenmenu");
				break;
			case TOKEN_PROGRESSBARSTYLE:
				token = xl_cheeselooks_gtk2_rc_parse_dummy (settings, scanner, "progressbarstyle");
				break;
			case TOKEN_MENUITEMSTYLE:
				token = xl_cheeselooks_gtk2_rc_parse_dummy (settings, scanner, "menuitemstyle");
				break;
			case TOKEN_LISTVIEWITEMSTYLE:
				token = xl_cheeselooks_gtk2_rc_parse_dummy (settings, scanner, "listviewitemstyle");
				break;

			default:
				g_scanner_get_next_token(scanner);
				token = G_TOKEN_RIGHT_CURLY;
				break;
		}

		if (token != G_TOKEN_NONE)
			return token;

		token = g_scanner_peek_next_token(scanner);
	}

	g_scanner_get_next_token(scanner);

	g_scanner_set_scope(scanner, old_scope);

	return G_TOKEN_NONE;
}

static void
xl_cheeselooks_rc_style_merge (GtkRcStyle *dest,
                           GtkRcStyle *src)
{
	xl_cheeselooksRcStyle *dest_w, *src_w;
	xl_cheeselooksRcFlags flags;

	xl_cheeselooks_parent_rc_class->merge (dest, src);

	if (!xl_cheeselooks_IS_RC_STYLE (src))
		return;

	src_w = xl_cheeselooks_RC_STYLE (src);
	dest_w = xl_cheeselooks_RC_STYLE (dest);

	flags = (~dest_w->flags) & src_w->flags;

	if (flags & CL_FLAG_STYLE)
		dest_w->style = src_w->style;
	if (flags & CL_FLAG_CONTRAST)
		dest_w->contrast = src_w->contrast;
	if (flags & CL_FLAG_RELIEFSTYLE)
		dest_w->reliefstyle = src_w->reliefstyle;
	if (flags & CL_FLAG_MENUBARSTYLE)
		dest_w->menubarstyle = src_w->menubarstyle;
	if (flags & CL_FLAG_TOOLBARSTYLE)
		dest_w->toolbarstyle = src_w->toolbarstyle;
	if (flags & CL_FLAG_FOCUS_COLOR)
		dest_w->focus_color = src_w->focus_color;
	if (flags & CL_FLAG_SCROLLBAR_COLOR)
		dest_w->scrollbar_color = src_w->scrollbar_color;
	if (flags & CL_FLAG_COLORIZE_SCROLLBAR)
		dest_w->colorize_scrollbar = src_w->colorize_scrollbar;
	if (flags & CL_FLAG_ANIMATION)
		dest_w->animation = src_w->animation;
	if (flags & CL_FLAG_RADIUS)
		dest_w->radius = src_w->radius;
	if (flags & CL_FLAG_HINT)
		dest_w->hint = src_w->hint;

	dest_w->flags |= src_w->flags;
}


/* Create an empty style suitable to this RC style
 */
static GtkStyle *
xl_cheeselooks_rc_style_create_style (GtkRcStyle *rc_style)
{
	return GTK_STYLE (g_object_new (xl_cheeselooks_TYPE_STYLE, NULL));
}
